@@ -6,6 +6,7 @@ require "singleton"
 require "timeout"
 require "pp"
 
+require "config"
 require "glib2"
 
 def debug(*args)
@@ -37,6 +38,10 @@ class Eat::Aria2 < GLib::Object
 
 	type_register
 
+	signal_new("connected", GLib::Signal::RUN_FIRST, nil,
+			nil)					# Return type: void
+	signal_new("disconnected", GLib::Signal::RUN_FIRST, nil,
+			nil)					# Return type: void
 	signal_new("download_status", GLib::Signal::RUN_FIRST, nil,
 			nil,					# Return type: void
 			String)					# Parameters: gid
@@ -50,6 +55,10 @@ class Eat::Aria2 < GLib::Object
 			nil,					# Return type: void
 			String)					# Parameters: gid
 
+	def signal_do_connected()
+	end
+	def signal_do_disconnected()
+	end
 	def signal_do_download_status(gid)
 	end
 	def signal_do_download_completed(gid)
@@ -95,31 +104,38 @@ class Eat::Aria2 < GLib::Object
 		return 0 if @use_local_server == false
 		begin
 			TCPSocket.new(@hostname, @port).close
+			debug("another service already listens on #{@port}")
 			return -1
 		rescue
 			begin
 				# Create config file
 				FileUtils.mkdir_p(@confdir, :mode => 0700)
 				FileUtils.touch(@confdir+"/aria2.conf")
+				# Save conf file with restricted options (e.g. daemon=false)
+				aria2_conf = Eat::Aria2Config.new(@confdir+"/aria2.conf")
+				aria2_conf.save
 				# Cleanup log file
 				FileUtils.rm(@confdir+"/aria2.log", :force => true)
 				# Launch aria2c process
-				command = "aria2c --disable-ipv6=true --enable-xml-rpc --xml-rpc-listen-port=#{@port} " \
+				command = "aria2c --disable-ipv6=true " \
+					"--enable-xml-rpc --xml-rpc-listen-port=#{@port} " \
 					"--conf-path=#{@confdir}/aria2.conf --log=#{@confdir}/aria2.log --log-level=notice " \
-					"--dht-file-path=#{@confdir}/dht.dat --lowest-speed-limit=1K"
+					"--dht-file-path=#{@confdir}/dht.dat"
 				debug("start server", command)
 				@@pid = Process.spawn(command, :pgroup=>true, :chdir=>ENV['HOME'],
 						STDOUT=>"/dev/null", STDIN=>"/dev/null")
 				Process.detach(@@pid)
 				# Wait for the server to respond properly to requests
-				begin
-					debug("test request")
+				Timeout::timeout(5) do
 					begin
-						result = @@client.call("aria2.getVersion")
-					rescue
-					end
-					sleep 1 if result == nil
-				end while result == nil
+						debug("test request")
+						begin
+							result = @@client.call("aria2.getVersion")
+						rescue
+						end
+						sleep 1 if result == nil
+					end while result == nil
+				end
 				# Store pid in a file
 				begin
 					File.open(@confdir+"/aria2.pid", "w").puts(@@pid)
@@ -129,6 +145,8 @@ class Eat::Aria2 < GLib::Object
 				puts "Started aria2 XML-RPC Server (pid #{@@pid.to_s})..."
 				return @@pid
 			rescue
+				debug("error start_server", $!)
+				@@pid = 0
 				return -2
 			end
 		end
@@ -142,17 +160,22 @@ class Eat::Aria2 < GLib::Object
 			debug("XMLRPC server didn't support the request")
 			puts e.message
 		rescue Errno::EPIPE => e
-			# Connection interrupted/timed out/server shutdown
+			# Connection interrupted
+			debug("EPIPE", $!)
+			# Try to reconnect if it was successfully connected previously
+			connect(true) if @is_connected
 			return nil if !@is_connected
-			connect(true)
 			result = call(method, args)
 		rescue Errno::ECONNREFUSED => e
 			# Connection refused
-			return nil if start_server < 0
+			debug("ECONNREFUSED", $!)
+			signal_emit("disconnected")
+			return nil if start_server <= 0
 			result = call(method, args)
 		rescue Exception => e
-			# Unhandled exception
-			puts $!, e.message
+			# Unhandled exception, no service listening/host unreachable/timed out/etc
+			debug("unhandled exception in call", $!)
+			signal_emit("disconnected")
 		end
 		result
 	end
@@ -168,17 +191,15 @@ class Eat::Aria2 < GLib::Object
 		return if @@client and force == false
 		@is_connected = false
 		begin
+			debug("connect")
 			@@client = XMLRPC::Client.new3({:host => @hostname, :path => "/rpc",
 					:port => @port, :user => @user, :password => @password,
 					:timeout => 60})
-			debug("call")
-			result = nil
-			Timeout::timeout(5) do
-				result = call("aria2.getVersion")
-			end
-			raise Exception.new("Unable to get an appropriate response to our request") if result == nil
+			result = call("aria2.getVersion")
+			raise Exception.new("Unable to connect to aria2 XML-RPC server") if result == nil
 			@version = result["version"]
 			@is_connected = true
+			signal_emit("connected")
 		rescue Exception => e
 			@@client = nil
 			puts "Can't establish a connection"
@@ -382,7 +403,6 @@ class Eat::Aria2Listener < Eat::Aria2
 				(@downloads - active_downloads).each do |gid|
 					# Get status
 					status = tell_status(gid)
-					status_msg = status["status"]
 					case status["status"]
 						when "complete" then signal_emit("download_completed", gid)
 						when "removed" then signal_emit("download_removed", gid)
@@ -393,7 +413,7 @@ class Eat::Aria2Listener < Eat::Aria2
 							else
 								puts "Unhandled error (%s: %s)" % [gid, status["errorCode"]]
 							end
-						else puts "Unhandled status (%s: %s)" % [gid, status_msg]
+						else puts "Unhandled status (%s: %s)" % [gid, status["status"]]
 					end
 				end
 
@@ -456,7 +476,7 @@ class Eat::Aria2Config
 		@values["load-cookies"] = "" # cookies file name to use, supports Firefox3 (sqlite) and Netscape
 		@values["save-cookies"] = "" # save cookies to file name in Netscape format
 		@values["use-head"] = false # use HEAD method for first request
-		@values["user-agent"] = "Eatmonkey/0.1.0 aria2/1.8.0"
+		@values["user-agent"] = "Eatmonkey/#{PACKAGE_VERSION} aria2"
 
 		# FTP options
 		@values["ftp-user"] = "anonymous" # FTP user
@@ -488,7 +508,7 @@ class Eat::Aria2Config
 		@values["listen-port"] = "6881-6999" # TCP listening port for BitTorrent
 		@values["max-overall-upload-limit"] = 0 # max overall upload speed in B/s, 0 means unlimited
 		@values["max-upload-limit"] = 0 # max upload speed per each download, 0 means unlimited
-		@values["peer-id-prefix"] = "Eatmonkey/0.1.0" # set peer ID, max 20 bytes
+		@values["peer-id-prefix"] = "Eatmonkey/#{PACKAGE_VERSION}" # set peer ID, max 20 bytes
 		@values["seed-ratio"] = 1.0 # specify share ratio
 
 		# Metalink options
@@ -542,6 +562,7 @@ class Eat::Aria2Config
 		begin
 			file = File.open(@filename)
 			while !file.eof and line = file.readline
+				next if line =~ /^daemon=.*$/
 				next if line !~ /^([a-zA-Z0-9-]+)=(.*)$/
 				key = $1
 				val = $2.strip
@@ -580,12 +601,16 @@ class Eat::Aria2Config
 		begin
 			file = File.open(@filename, "w")
 			@values.each do |key,val|
+				if key =~ /^(all-proxy|ca-certificate|certificate|http-proxy|https-proxy|private-key|ftp-proxy|dht-entry-point)$/
+					next if val.empty?
+				end
 				file.puts("#{key}=#{val}")
 			end
 			file.close
 		rescue
 			p $!
 		end
+		parse
 	end
 
 end
